<?php

namespace App\Domain\Cloud\Actions\Sync;

use App\Domain\Offers\Models\Category;
use App\Exceptions\ExceptionFormatter;
use Ensi\CloudApiSdk\Api\IndexesApi;
use Ensi\CloudApiSdk\Dto\Indexes\IndexesCategories\CategoryAction\CategoryAction;
use Ensi\CloudApiSdk\Dto\Indexes\IndexesCategories\CategoryAction\CategoryActionEnum;
use Ensi\CloudApiSdk\Dto\Indexes\IndexesCategories\CategoryAction\CategoryBody;
use Ensi\CloudApiSdk\Dto\Indexes\IndexesCategories\IndexesCategoriesRequest;
use Exception;
use Illuminate\Database\Eloquent\Collection;
use Psr\Log\LoggerInterface;
use Throwable;

class SyncCategoriesAction
{
    protected LoggerInterface $logger;

    protected array $categoryParentIds = [];

    public function __construct(protected IndexesApi $indexesApi)
    {
        $this->logger = logger()->channel('cloud:sync:categories');
    }

    /**
     * @param Collection<int,Category> $models
     *
     * @return void
     */
    public function execute(Collection $models): void
    {
        $this->categoryParentIds = Category::query()->pluck('parent_id', 'category_id')->all();

        $actions = [];
        $createdIds = [];
        $deletedIds = [];
        $this->logger->info("=== Prepare models");
        foreach ($models as $model) {
            $modelId = $model->category_id;
            $this->logger->info("Model: {$modelId}. Start prepare");

            try {
                $action = new CategoryAction();
                $action->id = (string)$modelId;

                if ($model->trashed() || !$model->isActiveForCloud()) {
                    if ($model->cloud_created) {
                        $action->action = CategoryActionEnum::DELETE;
                        $deletedIds[] = $modelId;
                    } else {
                        $this->logger->info("Model: {$modelId}. Skip, because model for delete, but not exist in cloud");

                        continue;
                    }
                } else {
                    if ($model->cloud_created) {
                        $action->action = CategoryActionEnum::UPDATE;
                    } else {
                        $action->action = CategoryActionEnum::CREATE;
                        $createdIds[] = $modelId;
                    }
                    $action->body = $this->cloudBodyFromModel($model);
                }

                $actions[$modelId] = $action;
                $this->logger->info("Model: {$modelId}. Added to sync. action = {$action->action->value}");
            } catch (Throwable $e) {
                $this->logger->error("Model: {$modelId}. " . ExceptionFormatter::line("Error", $e));

                continue;
            }
        }

        if (!$actions) {
            $this->logger->warning("No events for sync");

            return;
        }

        try {
            $this->send(array_values($actions));
        } catch (Exception $e) {
            $this->logger->error(ExceptionFormatter::line("Error on send", $e));

            return;
        }

        if ($createdIds) {
            $this->logger->info("Set cloud_created = true for models", [
                "category_ids" => $createdIds,
            ]);
            $count = Category::query()
                ->whereIn('category_id', $createdIds)
                ->update(['cloud_created' => true]);
            $this->logger->info("cloud_created = true setting: {$count}");
        }

        if ($deletedIds) {
            $this->logger->info("Set cloud_created = false for models", [
                "category_ids" => $deletedIds,
            ]);
            $count = Category::withTrashed()
                ->whereIn('category_id', $deletedIds)
                ->update(['cloud_created' => false]);
            $this->logger->info("cloud_created = false setting: {$count}");
        }
    }

    protected function cloudBodyFromModel(Category $model): CategoryBody
    {
        $body = new CategoryBody();
        $body->name = $model->name;

        $categoryIds = [];
        $currentId = $model->parent_id;
        while ($currentId) {
            $categoryIds[] = (string)$currentId;
            $currentId = $this->categoryParentIds[$currentId] ?? null;
        }
        $body->parent_ids = $categoryIds;

        return $body;
    }

    protected function send(array $actions): void
    {
        $request = new IndexesCategoriesRequest();
        $request->actions = $actions;

        $this->indexesApi->categories($request);
    }
}
