<?php

namespace App\Domain\Cloud\Actions\Sync;

use App\Domain\Offers\Models\Category;
use App\Domain\Offers\Models\Product;
use App\Exceptions\ExceptionFormatter;
use Ensi\CloudApiSdk\Api\IndexesApi;
use Ensi\CloudApiSdk\Dto\Indexes\IndexesProducts\IndexesProductsRequest;
use Ensi\CloudApiSdk\Dto\Indexes\IndexesProducts\ProductAction\ProductAction;
use Ensi\CloudApiSdk\Dto\Indexes\IndexesProducts\ProductAction\ProductActionEnum;
use Ensi\CloudApiSdk\Dto\Indexes\IndexesProducts\ProductAction\ProductBody;
use Ensi\CloudApiSdk\Dto\Indexes\IndexesProducts\ProductAction\ProductLocation;
use Ensi\CloudApiSdk\Dto\Indexes\IndexesProducts\ProductAction\ProductProperty;
use Ensi\LaravelEnsiFilesystem\Models\EnsiFile;
use Exception;
use Illuminate\Database\Eloquent\Collection;
use Psr\Log\LoggerInterface;
use Throwable;

class SyncProductsAction
{
    public const PROPERTY_MAIN_PRODUCT_NAME = 'Главный товар';
    public const PROPERTY_MAIN_PRODUCT_VALUE = 'да';

    protected LoggerInterface $logger;

    public function __construct(protected IndexesApi $indexesApi)
    {
        $this->logger = logger()->channel('cloud:sync:products');
    }

    /**
     * @param Collection<int,Product> $models
     *
     * @return void
     */
    public function execute(Collection $models): void
    {
        $models->loadMissing([
            'offers',
            'productPropertyValues.property',
            'productPropertyValues.directoryValue',
            'brand',
            'productGroup',
            'categories',
        ]);
        $actions = [];
        $createdIds = [];
        $deletedIds = [];
        $this->logger->info("=== Prepare models");
        foreach ($models as $model) {
            $modelId = $model->product_id;
            $this->logger->info("Model: {$modelId}. Start prepare");

            try {
                $action = new ProductAction();
                $action->id = (string)$modelId;

                if ($model->trashed() || !$model->allow_publish) {
                    if ($model->cloud_created) {
                        $action->action = ProductActionEnum::DELETE;
                        $deletedIds[] = $modelId;
                    } else {
                        $this->logger->info("Model: {$modelId}. Skip, because model for delete, but not exist in cloud");

                        continue;
                    }
                } else {
                    if ($model->cloud_created) {
                        $action->action = ProductActionEnum::UPDATE;
                    } else {
                        $action->action = ProductActionEnum::CREATE;
                        $createdIds[] = $modelId;
                    }
                    $action->body = $this->cloudBodyFromModel($model);
                }

                $actions[$modelId] = $action;
                $this->logger->info("Model: {$modelId}. Added to sync. action = {$action->action->value}");
            } catch (Throwable $e) {
                $this->logger->error("Model: {$modelId}. " . ExceptionFormatter::line("Error", $e));

                continue;
            }
        }

        if (!$actions) {
            $this->logger->warning("No events for sync");

            return;
        }

        try {
            $this->send(array_values($actions));
        } catch (Exception $e) {
            $this->logger->error(ExceptionFormatter::line("Error on send", $e));

            return;
        }

        if ($createdIds) {
            $this->logger->info("Set cloud_created = true for models", [
                "product_ids" => $createdIds,
            ]);
            $count = Product::query()
                ->whereIn('product_id', $createdIds)
                ->update(['cloud_created' => true]);
            $this->logger->info("cloud_created = true setting: {$count}");
        }

        if ($deletedIds) {
            $this->logger->info("Set cloud_created = false for models", [
                "product_ids" => $deletedIds,
            ]);
            $count = Product::withTrashed()
                ->whereIn('product_id', $deletedIds)
                ->update(['cloud_created' => false]);
            $this->logger->info("cloud_created = false setting: {$count}");
        }
    }

    protected function cloudBodyFromModel(Product $model): ProductBody
    {
        $body = new ProductBody();
        $body->name = $model->name;
        $body->category_ids = $model->categories->map(fn (Category $category) => (string)$category->category_id)->toArray();
        if ($model->brand) {
            $body->brand = $model->brand->name;
        }
        $body->vendor_code = $model->vendor_code;
        $body->barcodes = $model->barcode ? [$model->barcode] : [];
        if ($model->description) {
            $body->description = $model->description;
        }
        if ($model->main_image_file) {
            $body->picture = EnsiFile::public($model->main_image_file)->getUrl();
        }

        $body->group_ids = [];
        $isProductMain = true;
        if ($model->productGroup?->is_active) {
            $body->group_ids = [(string)$model->productGroup->product_group_id];
            if ($model->productGroup->main_product_id != $model->product_id) {
                $isProductMain = false;
            }

        }

        $locations = [];
        foreach ($model->offers as $offer) {
            $location = new ProductLocation();
            $location->id = '1'; // hardcode from Analytics
            $price = $offer->price ?? $offer->base_price;
            if ($price) {
                $location->price = $price;
            }
            $locations[] = $location;

            break;
        }
        if (!$locations) {
            throw new Exception("No offers");
        }
        $body->locations = $locations;

        $propertyValues = [];
        foreach ($model->productPropertyValues as $propertyValue) {
            $property = $propertyValue->property;
            if (!$property || !$property->is_public || !$property->is_active) {
                continue;
            }
            $propValueAndName = $propertyValue->getPropValueAndName();
            if (!$propValueAndName) {
                continue;
            }

            $propertyValues[$property->name][] = $propValueAndName[0];
        }
        $properties = [];
        foreach ($propertyValues as $name => $values) {
            $property = new ProductProperty();
            $property->name = (string)$name;
            $property->values = array_map('strval', $values);

            $properties[] = $property;
        }
        if ($isProductMain) {
            $property = new ProductProperty();
            $property->name = static::PROPERTY_MAIN_PRODUCT_NAME;
            $property->values = [static::PROPERTY_MAIN_PRODUCT_VALUE];

            $properties[] = $property;
        }

        $body->properties = $properties;

        return $body;
    }

    protected function send(array $actions): void
    {
        $request = new IndexesProductsRequest();
        $request->actions = $actions;

        $this->indexesApi->products($request);
    }
}
