<?php

namespace App\Domain\Cloud\Actions\Sync;

use App\Domain\Offers\Models\ActualCategoryProperty;
use App\Domain\Offers\Models\Property;
use App\Exceptions\ExceptionFormatter;
use Ensi\CloudApiSdk\Api\IndexesApi;
use Ensi\CloudApiSdk\Dto\Indexes\IndexesProperties\IndexesPropertiesRequest;
use Ensi\CloudApiSdk\Dto\Indexes\IndexesProperties\PropertyAction\PropertyAction;
use Ensi\CloudApiSdk\Dto\Indexes\IndexesProperties\PropertyAction\PropertyActionEnum;
use Ensi\CloudApiSdk\Dto\Indexes\IndexesProperties\PropertyAction\PropertyBody;
use Exception;
use Illuminate\Database\Eloquent\Collection;
use Psr\Log\LoggerInterface;
use Throwable;

class SyncPropertiesAction
{
    protected LoggerInterface $logger;

    public function __construct(protected IndexesApi $indexesApi)
    {
        $this->logger = logger()->channel('cloud:sync:properties');
    }

    /**
     * @param Collection<int,Property> $models
     *
     * @return void
     */
    public function execute(Collection $models): void
    {
        $models->loadMissing([
            'actualCategoryProperties.category',
        ]);
        $actions = [];
        $createdIds = [];
        $deletedIds = [];
        $this->logger->info("=== Prepare models");
        foreach ($models as $model) {
            $modelId = $model->property_id;
            $this->logger->info("Model: {$modelId}. Start prepare");

            try {
                $action = new PropertyAction();
                $action->id = (string)$modelId;
                $actualCategoryProperties = $model->actualCategoryProperties->filter(function ($link) {
                    /** @var ActualCategoryProperty $link */
                    return $link->category?->isActiveForCloud() ?: false;
                });

                if ($model->trashed() || !$model->is_active || $actualCategoryProperties->isEmpty()) {
                    if ($model->cloud_created) {
                        $action->action = PropertyActionEnum::DELETE;
                        $deletedIds[] = $modelId;
                    } else {
                        $this->logger->info("Model: {$modelId}. Skip, because model for delete, but not exist in cloud");

                        continue;
                    }
                } else {
                    if ($model->cloud_created) {
                        $action->action = PropertyActionEnum::UPDATE;
                    } else {
                        $action->action = PropertyActionEnum::CREATE;
                        $createdIds[] = $modelId;
                    }
                    $action->body = $this->cloudBodyFromModel($model, $actualCategoryProperties);
                }

                $actions[$modelId] = $action;
                $this->logger->info("Model: {$modelId}. Added to sync. action = {$action->action->value}");
            } catch (Throwable $e) {
                $this->logger->error("Model: {$modelId}. " . ExceptionFormatter::line("Error", $e));

                continue;
            }
        }

        if (!$actions) {
            $this->logger->warning("No events for sync");

            return;
        }

        try {
            $this->send(array_values($actions));
        } catch (Exception $e) {
            $this->logger->error(ExceptionFormatter::line("Error on send", $e));

            return;
        }

        if ($createdIds) {
            $this->logger->info("Set cloud_created = true for models", [
                "property_ids" => $createdIds,
            ]);
            $count = Property::query()
                ->whereIn('property_id', $createdIds)
                ->update(['cloud_created' => true]);
            $this->logger->info("cloud_created = true setting: {$count}");
        }

        if ($deletedIds) {
            $this->logger->info("Set cloud_created = false for models", [
                "property_ids" => $deletedIds,
            ]);
            $count = Property::withTrashed()
                ->whereIn('property_id', $deletedIds)
                ->update(['cloud_created' => false]);
            $this->logger->info("cloud_created = false setting: {$count}");
        }
    }

    protected function cloudBodyFromModel(Property $model, Collection $actualCategoryProperties): PropertyBody
    {
        $body = new PropertyBody();
        $body->name = $model->name;
        $body->category_ids = array_map('strval', $actualCategoryProperties->pluck('category_id')->values()->all());

        return $body;
    }

    protected function send(array $actions): void
    {
        $request = new IndexesPropertiesRequest();
        $request->actions = $actions;

        $this->indexesApi->properties($request);
    }
}
