<?php

namespace App\Domain\Cloud\Actions\Sync;

use App\Domain\Offers\Models\PropertyDirectoryValue;
use App\Exceptions\ExceptionFormatter;
use Ensi\CloudApiSdk\Api\IndexesApi;
use Ensi\CloudApiSdk\Dto\Indexes\IndexesPropertyDirectoryValues\IndexesPropertyDirectoryValuesRequest;
use Ensi\CloudApiSdk\Dto\Indexes\IndexesPropertyDirectoryValues\PropertyDirectoryValueAction\PropertyDirectoryValueAction;
use Ensi\CloudApiSdk\Dto\Indexes\IndexesPropertyDirectoryValues\PropertyDirectoryValueAction\PropertyDirectoryValueActionEnum;
use Ensi\CloudApiSdk\Dto\Indexes\IndexesPropertyDirectoryValues\PropertyDirectoryValueAction\PropertyDirectoryValueBody;
use Exception;
use Illuminate\Database\Eloquent\Collection;
use Psr\Log\LoggerInterface;
use Throwable;

class SyncPropertyDirectoryValuesAction
{
    protected LoggerInterface $logger;

    public function __construct(protected IndexesApi $indexesApi)
    {
        $this->logger = logger()->channel('cloud:sync:property-directory-values');
    }

    /**
     * @param Collection<int,PropertyDirectoryValue> $models
     *
     * @return void
     */
    public function execute(Collection $models): void
    {
        $actions = [];
        $createdIds = [];
        $deletedIds = [];
        $this->logger->info("=== Prepare models");
        foreach ($models as $model) {
            $modelId = $model->directory_value_id;
            $this->logger->info("Model: {$modelId}. Start prepare");

            try {
                $action = new PropertyDirectoryValueAction();
                $action->id = (string)$modelId;

                if ($model->trashed()) {
                    if ($model->cloud_created) {
                        $action->action = PropertyDirectoryValueActionEnum::DELETE;
                        $deletedIds[] = $modelId;
                    } else {
                        $this->logger->info("Model: {$modelId}. Skip, because model for delete, but not exist in cloud");

                        continue;
                    }
                } else {
                    if ($model->cloud_created) {
                        $action->action = PropertyDirectoryValueActionEnum::UPDATE;
                    } else {
                        $action->action = PropertyDirectoryValueActionEnum::CREATE;
                        $createdIds[] = $modelId;
                    }
                    $action->body = $this->cloudBodyFromModel($model);
                }

                $actions[$modelId] = $action;
                $this->logger->info("Model: {$modelId}. Added to sync. action = {$action->action->value}");
            } catch (Throwable $e) {
                $this->logger->error("Model: {$modelId}. " . ExceptionFormatter::line("Error", $e));

                continue;
            }
        }

        if (!$actions) {
            $this->logger->warning("No events for sync");

            return;
        }

        try {
            $this->send(array_values($actions));
        } catch (Exception $e) {
            $this->logger->error(ExceptionFormatter::line("Error on send", $e));

            return;
        }

        if ($createdIds) {
            $this->logger->info("Set cloud_created = true for models", [
                "directory_value_ids" => $createdIds,
            ]);
            $count = PropertyDirectoryValue::query()
                ->whereIn('directory_value_id', $createdIds)
                ->update(['cloud_created' => true]);
            $this->logger->info("cloud_created = true setting: {$count}");
        }

        if ($deletedIds) {
            $this->logger->info("Set cloud_created = false for models", [
                "directory_value_ids" => $deletedIds,
            ]);
            $count = PropertyDirectoryValue::withTrashed()
                ->whereIn('directory_value_id', $deletedIds)
                ->update(['cloud_created' => false]);
            $this->logger->info("cloud_created = false setting: {$count}");
        }
    }

    protected function cloudBodyFromModel(PropertyDirectoryValue $model): PropertyDirectoryValueBody
    {
        $body = new PropertyDirectoryValueBody();
        $body->name = $model->name;
        $body->property_id = (string)$model->property_id;

        return $body;
    }

    protected function send(array $actions): void
    {
        $request = new IndexesPropertyDirectoryValuesRequest();
        $request->actions = $actions;

        $this->indexesApi->propertyDirectoryValues($request);
    }
}
