<?php

use App\Domain\Cloud\Jobs\SyncPropertiesJob;
use App\Domain\Cloud\Jobs\Tests\TestCases\SyncPropertiesJobTestCase;
use App\Domain\Offers\Models\Property;
use Ensi\CloudApiSdk\Dto\Indexes\IndexesProperties\PropertyAction\PropertyActionEnum;
use Illuminate\Database\Eloquent\Collection;

use function Pest\Laravel\assertDatabaseHas;
use function PHPUnit\Framework\assertArrayNotHasKey;
use function PHPUnit\Framework\assertEquals;

uses(SyncPropertiesJobTestCase::class);
uses()->group('component', 'SyncPropertiesJob');

test("Action SyncPropertiesJob integration disable", function () {
    /** @var SyncPropertiesJobTestCase $this */
    $this->cloudIntegrationDisable();

    /** @var Collection<Property> $properties */
    $properties = Property::factory()->count(2)->create();

    $this->neverSendProperty();

    SyncPropertiesJob::dispatchSync($properties->pluck('property_id')->all());
});

test("Action SyncPropertiesJob create|update success", function (bool $cloudCreated) {
    /** @var SyncPropertiesJobTestCase $this */

    /** @var Property $property */
    $property = Property::factory()->active()->create(['cloud_created' => $cloudCreated]);
    $this->createLink($property->property_id, categoryActive: true);

    $action = &$this->shouldSendProperty();

    SyncPropertiesJob::dispatchSync([$property->property_id]);

    assertEquals($cloudCreated ? PropertyActionEnum::UPDATE : PropertyActionEnum::CREATE, $action->action);

    assertDatabaseHas(Property::class, [
        'id' => $property->id,
        'cloud_created' => true,
    ]);
})->with([true, false]);

test("Action SyncPropertiesJob base fields success", function () {
    /** @var SyncPropertiesJobTestCase $this */

    /** @var Property $property */
    $property = Property::factory()->active()->create();

    /// === CREATE LINKS

    $linkSuccess = $this->createLink($property->property_id, categoryActive: true);
    $this->createLink($property->property_id, categoryInactive: true);
    $this->createLink($property->property_id);

    /// === END CREATE LINKS

    $action = &$this->shouldSendProperty();

    SyncPropertiesJob::dispatchSync([$property->property_id]);

    expect($action->id)->toEqual($property->property_id)->toBeString();

    $body = $action->body->toArray();
    assertEquals($property->name, $action->body->name);

    expect($action->body->category_ids)->toEqualCanonicalizing([$linkSuccess->category_id]);
    foreach ($action->body->category_ids as $value) {
        expect($value)->toBeString();
    }

    assertArrayNotHasKey('type', $body);
});

test("Action SyncPropertiesJob delete success", function (bool $cloudCreated) {
    /** @var SyncPropertiesJobTestCase $this */

    /** @var Property $property */
    $property = Property::factory()->trashed()->create([
        'cloud_created' => $cloudCreated,
    ]);

    if ($cloudCreated) {
        $action = &$this->shouldSendProperty();
    } else {
        $this->neverSendProperty();
    }

    SyncPropertiesJob::dispatchSync([$property->property_id]);

    if ($cloudCreated) {
        assertEquals(PropertyActionEnum::DELETE, $action->action);
        assertEquals($property->property_id, $action->id);
    }

    assertDatabaseHas(Property::class, [
        'id' => $property->id,
        'cloud_created' => false,
    ]);
})->with([true, false]);

test("Action SyncPropertiesJob active success", function (
    bool $initCloudCreated,
    bool $active,
    ?PropertyActionEnum $actionType,
    bool $resultCloudCreated,
) {
    /** @var SyncPropertiesJobTestCase $this */

    /** @var Property $property */
    $property = Property::factory()->active($active)->create([
        'cloud_created' => $initCloudCreated,
    ]);
    $this->createLink($property->property_id, categoryActive: true);

    $action = null;
    if (is_null($actionType)) {
        $this->neverSendProperty();
    } else {
        $action = &$this->shouldSendProperty();
    }

    SyncPropertiesJob::dispatchSync([$property->property_id]);

    if (!is_null($actionType)) {
        assertEquals($actionType, $action->action);
        assertEquals($property->property_id, $action->id);
    }

    assertDatabaseHas(Property::class, [
        'id' => $property->id,
        'cloud_created' => $resultCloudCreated,
    ]);
})->with([
    [false, true, PropertyActionEnum::CREATE, true],
    [true, true, PropertyActionEnum::UPDATE, true],
    [false, false, null, false],
    [true, false, PropertyActionEnum::DELETE, false],
]);

test("Action SyncPropertiesJob link success", function (
    bool $initCloudCreated,
    ?string $withLink,
    ?PropertyActionEnum $actionType,
    bool $resultCloudCreated,
) {
    /** @var SyncPropertiesJobTestCase $this */

    /** @var Property $property */
    $property = Property::factory()->active()->create([
        'cloud_created' => $initCloudCreated,
    ]);
    if ($withLink) {
        if ($withLink == 'success') {
            $this->createLink($property->property_id, categoryActive: true);
        } elseif ($withLink == 'no_active') {
            $this->createLink($property->property_id, categoryInactive: true);
        } elseif ($withLink == 'no_category') {
            $this->createLink($property->property_id);
        }
    }

    $action = null;
    if (is_null($actionType)) {
        $this->neverSendProperty();
    } else {
        $action = &$this->shouldSendProperty();
    }

    SyncPropertiesJob::dispatchSync([$property->property_id]);

    if (!is_null($actionType)) {
        assertEquals($actionType, $action->action);
        assertEquals($property->property_id, $action->id);
    }

    assertDatabaseHas(Property::class, [
        'id' => $property->id,
        'cloud_created' => $resultCloudCreated,
    ]);
})->with([
    [false, 'success', PropertyActionEnum::CREATE, true],
    [true, 'success', PropertyActionEnum::UPDATE, true],
    [false, null, null, false],
    [true, null, PropertyActionEnum::DELETE, false],
    [true, 'no_active', PropertyActionEnum::DELETE, false],
    [true, 'no_category', PropertyActionEnum::DELETE, false],
]);
