<?php

use App\Domain\Kafka\Actions\Listen\ListenActualCategoryPropertyAction;
use App\Domain\Kafka\Messages\Listen\ModelEvent\ActualCategoryProperty\ActualCategoryPropertyEventMessage;
use App\Domain\Kafka\Messages\Listen\ModelEvent\ModelEventMessage;
use App\Domain\Offers\Models\ActualCategoryProperty;
use App\Domain\Offers\Models\Property;
use Ensi\LaravelTestFactories\FakerProvider;

use function Pest\Laravel\assertDatabaseHas;
use function Pest\Laravel\assertDatabaseMissing;
use function PHPUnit\Framework\assertEquals;
use function PHPUnit\Framework\assertGreaterThan;

use Tests\IntegrationTestCase;

uses(IntegrationTestCase::class);
uses()->group('integration', 'kafka', 'kafka-listen-actual-category-property');

test("Action ListenActualCategoryPropertyAction create success", function () {
    /** @var IntegrationTestCase $this */
    $actualCategoryPropertyId = 1;
    $propertyId = 1;
    $categoryId = 1;
    $message = ActualCategoryPropertyEventMessage::factory()
        ->attributes(['id' => $actualCategoryPropertyId, 'property_id' => $propertyId, 'category_id' => $categoryId])
        ->event(ModelEventMessage::CREATE)
        ->make();

    // Create property that's linked to category and must be marked for cloud
    $propertyMark = Property::factory()->create(['property_id' => $propertyId]);
    // Create another property, to ensure it does not get marked for cloud
    $propertyOther = Property::factory()->create(['property_id' => $propertyId + 1]);

    resolve(ListenActualCategoryPropertyAction::class)->execute($message);

    assertDatabaseHas(ActualCategoryProperty::class, [
        'actual_category_property_id' => $actualCategoryPropertyId,
    ]);
    assertNewModelFieldGreaterThan($propertyMark, 'cloud_fields_updated_at');
    assertNewModelFieldEquals($propertyOther, 'cloud_fields_updated_at');
});

test("Action ListenActualCategoryPropertyAction update success", function (array $dirty, bool $isDirty) {
    /** @var IntegrationTestCase $this */
    /** @var ActualCategoryProperty $actualCategoryProperty */
    $actualCategoryProperty = ActualCategoryProperty::factory()->create();
    $oldUpdatedAt = $actualCategoryProperty->updated_at;
    $newUpdatedAt = $actualCategoryProperty->updated_at->subDay();
    $message = ActualCategoryPropertyEventMessage::factory()
        ->forModel($actualCategoryProperty)
        ->attributes([
            'category_id' => $actualCategoryProperty->category_id + 1,
            'updated_at' => $newUpdatedAt->toJSON(),
        ])
        ->event(ModelEventMessage::UPDATE)
        ->make(['dirty' => $dirty]);

    resolve(ListenActualCategoryPropertyAction::class)->execute($message);

    // updated_at is not dependent on model
    $actualCategoryProperty->refresh();
    if ($isDirty) {
        assertGreaterThan($newUpdatedAt, $actualCategoryProperty->updated_at);
    } else {
        assertEquals($oldUpdatedAt, $actualCategoryProperty->updated_at);
    }
})->with([
    [['category_id'], true],
    [['is_required'], false],
]);

test("Action ListenActualCategoryPropertyAction delete success", function (?bool $always) {
    FakerProvider::$optionalAlways = $always;

    /** @var IntegrationTestCase $this */
    /** @var ActualCategoryProperty $actualCategoryProperty */
    $actualCategoryProperty = ActualCategoryProperty::factory()->create();
    $message = ActualCategoryPropertyEventMessage::factory()
        ->forModel($actualCategoryProperty)
        ->event(ModelEventMessage::DELETE)
        ->make();

    // Create product that's linked to category and has needed property and must be marked for cloud
    $propertyMark = Property::factory()->create(['property_id' => $actualCategoryProperty->property_id]);
    // Create another property, to ensure it does not get marked for cloud
    $propertyOther = Property::factory()->create(['property_id' => $actualCategoryProperty->property_id + 1]);

    resolve(ListenActualCategoryPropertyAction::class)->execute($message);

    assertDatabaseMissing(ActualCategoryProperty::class, ['actual_category_property_id' => $actualCategoryProperty->actual_category_property_id]);

    assertNewModelFieldGreaterThan($propertyMark, 'cloud_fields_updated_at');
    assertNewModelFieldEquals($propertyOther, 'cloud_fields_updated_at');
})->with(FakerProvider::$optionalDataset);
