<?php

use App\Domain\Kafka\Actions\Listen\ListenPropertyDirectoryValueAction;
use App\Domain\Kafka\Actions\Tests\Factories\ProductLinkedFactory;
use App\Domain\Kafka\Messages\Listen\ModelEvent\ModelEventMessage;
use App\Domain\Kafka\Messages\Listen\ModelEvent\PropertyDirectoryValue\PropertyDirectoryValueEventMessage;
use App\Domain\Offers\Models\PropertyDirectoryValue;
use Ensi\LaravelTestFactories\FakerProvider;

use function Pest\Laravel\assertDatabaseHas;
use function Pest\Laravel\isSoftDeletableModel;
use function PHPUnit\Framework\assertEquals;

use Tests\IntegrationTestCase;

uses(IntegrationTestCase::class);
uses()->group('integration', 'kafka', 'kafka-listen-property-directory-value');

test("Action ListenPropertyDirectoryValueAction create success", function (?bool $always) {
    /** @var IntegrationTestCase $this */
    FakerProvider::$optionalAlways = $always;

    $propertyDirectoryValueId = 1;
    $message = PropertyDirectoryValueEventMessage::factory()
        ->attributes(['id' => $propertyDirectoryValueId])
        ->event(ModelEventMessage::CREATE)
        ->make();

    // product that is associated with this model and should be updated with cloud_fields_updated_at
    $productMark = ProductLinkedFactory::createFromDirectoryValue($propertyDirectoryValueId);
    // other product
    $productOther = ProductLinkedFactory::createFromDirectoryValue($propertyDirectoryValueId + 1);

    $now = now();
    Date::setTestNow($now);
    resolve(ListenPropertyDirectoryValueAction::class)->execute($message);

    assertDatabaseHas(PropertyDirectoryValue::class, [
        'directory_value_id' => $propertyDirectoryValueId,
        'cloud_fields_updated_at' => $now->format((new PropertyDirectoryValue())->getDateFormat()),
    ]);
    assertNewModelFieldGreaterThan($productMark, 'cloud_fields_updated_at');
    assertNewModelFieldEquals($productOther, 'cloud_fields_updated_at');
})->with(FakerProvider::$optionalDataset);

test("Action ListenPropertyDirectoryValueAction update success", function (
    string $updatedField,
    mixed $newValue,
    bool $isUpdated,
    bool $isCloudUpdated,
    bool $isCloudProductUpdated
) {
    /** @var IntegrationTestCase $this */

    /** @var PropertyDirectoryValue $propertyDirectoryValue */
    $propertyDirectoryValue = PropertyDirectoryValue::factory()->create();
    $updatedAt = $propertyDirectoryValue->updated_at->addDay();
    $message = PropertyDirectoryValueEventMessage::factory()
        ->forModel($propertyDirectoryValue)
        ->attributes([$updatedField => $newValue, 'updated_at' => $updatedAt->toJSON()])
        ->event(ModelEventMessage::UPDATE)
        ->make(['dirty' => [$updatedField]]);

    // product that is associated with this model and should be updated with cloud_fields_updated_at
    $productMark = ProductLinkedFactory::createFromDirectoryValue($propertyDirectoryValue->directory_value_id);
    // other product
    $productOther = ProductLinkedFactory::createFromDirectoryValue($propertyDirectoryValue->directory_value_id + 1);

    resolve(ListenPropertyDirectoryValueAction::class)->execute($message);

    if ($isCloudUpdated) {
        assertNewModelFieldGreaterThan($propertyDirectoryValue, 'cloud_fields_updated_at');
    } else {
        assertNewModelFieldEquals($propertyDirectoryValue, 'cloud_fields_updated_at');
    }
    $needUpdatedAt = $isUpdated ? $updatedAt : $propertyDirectoryValue->updated_at;
    $needField = $isUpdated ? $newValue : $propertyDirectoryValue->getAttribute($updatedField);
    $propertyDirectoryValue->refresh();
    assertEquals($needUpdatedAt, $propertyDirectoryValue->updated_at);
    assertEquals($needField, $propertyDirectoryValue->getAttribute($updatedField));

    // Verification is similar to creation, but if there is no saving, then there should be no marking
    if ($isCloudProductUpdated) {
        assertNewModelFieldGreaterThan($productMark, 'cloud_fields_updated_at');
    } else {
        assertNewModelFieldEquals($productMark, 'cloud_fields_updated_at');
    }
    assertNewModelFieldEquals($productOther, 'cloud_fields_updated_at');
})->with([
    ['property_id', 33, true, true, false],
    ['name', 'Новое имя', true, true, false],
    ['code', 'new_code', true, false, false],
    ['value', 'new_value', true, false, true],
    ['type', 1, true, false, false],
    ['other_field', 'value', false, false, false],
]);

test("Action ListenPropertyDirectoryValueAction delete success", function () {
    /** @var IntegrationTestCase $this */

    /** @var PropertyDirectoryValue $propertyDirectoryValue */
    $propertyDirectoryValue = PropertyDirectoryValue::factory()->create();
    $message = PropertyDirectoryValueEventMessage::factory()
        ->forModel($propertyDirectoryValue)
        ->event(ModelEventMessage::DELETE)
        ->make();

    // product that is associated with this property and should not be updated with cloud_fields_updated_at
    // because no mark on delete
    $product = ProductLinkedFactory::createFromDirectoryValue($propertyDirectoryValue->directory_value_id);

    resolve(ListenPropertyDirectoryValueAction::class)->execute($message);

    isSoftDeletableModel($propertyDirectoryValue);
    assertNewModelFieldGreaterThan($propertyDirectoryValue, 'cloud_fields_updated_at');
    // product must not be marked
    assertNewModelFieldEquals($product, 'cloud_fields_updated_at');
});
