<?php

use App\Domain\Kafka\Actions\Listen\ListenPropertyAction;
use App\Domain\Kafka\Actions\Tests\Factories\ProductLinkedFactory;
use App\Domain\Kafka\Messages\Listen\ModelEvent\ModelEventMessage;
use App\Domain\Kafka\Messages\Listen\ModelEvent\Property\PropertyEventMessage;
use App\Domain\Offers\Models\Property;
use Ensi\LaravelTestFactories\FakerProvider;
use Illuminate\Support\Facades\Date;

use function Pest\Laravel\assertDatabaseHas;
use function Pest\Laravel\isSoftDeletableModel;
use function PHPUnit\Framework\assertEquals;

use Tests\IntegrationTestCase;

uses(IntegrationTestCase::class);
uses()->group('integration', 'kafka', 'kafka-listen-property');

test("Action ListenPropertyAction create success", function (?bool $always) {
    /** @var IntegrationTestCase $this */
    FakerProvider::$optionalAlways = $always;

    $propertyId = 1;
    $message = PropertyEventMessage::factory()
        ->attributes(['id' => $propertyId])
        ->event(ModelEventMessage::CREATE)
        ->make();

    // product that is associated with this model and should be updated with cloud_fields_updated_at
    $productMark = ProductLinkedFactory::createFromProperty($propertyId);
    // other product
    $productOther = ProductLinkedFactory::createFromProperty($propertyId + 1);

    $now = now();
    Date::setTestNow($now);
    resolve(ListenPropertyAction::class)->execute($message);

    assertDatabaseHas(Property::class, [
        'property_id' => $propertyId,
        'cloud_fields_updated_at' => $now->format((new Property())->getDateFormat()),
    ]);
    assertNewModelFieldGreaterThan($productMark, 'cloud_fields_updated_at');
    assertNewModelFieldEquals($productOther, 'cloud_fields_updated_at');
})->with(FakerProvider::$optionalDataset);

test("Action ListenPropertyAction update success", function (
    string $updatedField,
    mixed $newValue,
    bool $isUpdated,
    bool $isCloudUpdated,
    bool $isCloudProductUpdated,
    ?string $modelField = null,
) {
    /** @var IntegrationTestCase $this */

    /** @var Property $property */
    $property = Property::factory()->create(['is_public' => false, 'is_active' => false]);
    $updatedAt = $property->updated_at->addDay();
    $message = PropertyEventMessage::factory()
        ->forModel($property)
        ->attributes([$updatedField => $newValue, 'updated_at' => $updatedAt->toJSON()])
        ->event(ModelEventMessage::UPDATE)
        ->make(['dirty' => [$updatedField]]);

    // product that is associated with this property and should be updated with cloud_fields_updated_at
    $productMark = ProductLinkedFactory::createFromProperty($property->property_id);
    // other product
    $productOther = ProductLinkedFactory::createFromProperty($property->property_id + 1);

    resolve(ListenPropertyAction::class)->execute($message);

    if ($isCloudUpdated) {
        assertNewModelFieldGreaterThan($property, 'cloud_fields_updated_at');
    } else {
        assertNewModelFieldEquals($property, 'cloud_fields_updated_at');
    }
    $needUpdatedAt = $isUpdated ? $updatedAt : $property->updated_at;
    $modelField = $modelField ?? $updatedField;
    $needField = $isUpdated ? $newValue : $property->getAttribute($modelField);
    $property->refresh();
    assertEquals($needUpdatedAt, $property->updated_at);
    assertEquals($needField, $property->getAttribute($modelField));

    // Verification is similar to creation, but if there is no saving, then there should be no marking
    if ($isCloudProductUpdated) {
        assertNewModelFieldGreaterThan($productMark, 'cloud_fields_updated_at');
    } else {
        assertNewModelFieldEquals($productMark, 'cloud_fields_updated_at');
    }
    assertNewModelFieldEquals($productOther, 'cloud_fields_updated_at');
})->with([
    ['display_name', 'Новое имя', true, true, true, 'name'],
    ['code', 'new_code', true, false, false],
    ['type', 'string', true, false, false],
    ['is_public', true, true, false, true],
    ['is_active', true, true, true, true],
    ['other_field', 'value', false, false, false],
]);

test("Action ListenPropertyAction delete success", function () {
    /** @var IntegrationTestCase $this */

    /** @var Property $property */
    $property = Property::factory()->create();
    $message = PropertyEventMessage::factory()
        ->forModel($property)
        ->event(ModelEventMessage::DELETE)
        ->make();

    // product that is associated with this property and should not be updated with cloud_fields_updated_at
    // because no mark on delete
    $product = ProductLinkedFactory::createFromProperty($property->property_id);

    resolve(ListenPropertyAction::class)->execute($message);

    isSoftDeletableModel($property);
    assertNewModelFieldGreaterThan($property, 'cloud_fields_updated_at');
    // product must not be marked
    assertNewModelFieldEquals($product, 'cloud_fields_updated_at');
});
